package com.flyfish.framework.compiler.support;

import lombok.Builder;
import lombok.Data;
import lombok.NoArgsConstructor;
import lombok.experimental.Accessors;
import org.apache.commons.lang3.StringUtils;

/**
 * java源代码对象，可以继承进行扩展
 * @author wangyu
 */
@Data
@NoArgsConstructor(staticName = "create")
@Accessors(chain = true)
public class JavaSource {

    // 类名
    private String className;

    // 源码
    private String source;

    // 包名
    private String packageName;

    // 父类名
    private String superClass;

    // bean的class
    private String beanClass;

    // 查询实体class
    private String queryBeanClass;

    // controller需要，提供请求地址
    private String uri;

    public String getSuperClassName() {
        return StringUtils.substringAfterLast(superClass, ".");
    }

    public String getBeanClassName() {
        return StringUtils.substringAfterLast(beanClass, ".");
    }

    public String getQueryBeanClassName() {
        return StringUtils.substringAfterLast(queryBeanClass, ".");
    }

    // 获取限定查询实体的后缀
    public String getQueryBeanSuffix() {
        String beanClassName = getBeanClassName();
        return queryBeanClass.contains(beanClassName) ? "" : "<" + beanClassName + ">";
    }
}
