package com.flyfish.framework.config;

import com.flyfish.framework.bean.EnumValue;
import com.flyfish.framework.enums.NamedEnum;
import com.flyfish.framework.utils.StringFormats;
import org.apache.commons.lang3.ClassUtils;
import org.reflections.Reflections;
import org.springframework.beans.factory.InitializingBean;
import org.springframework.context.annotation.ImportBeanDefinitionRegistrar;

import java.util.*;
import java.util.stream.Collectors;

/**
 * 枚举相关，专门处理枚举
 * @author wangyu
 */
public class EnumConfig implements InitializingBean, ImportBeanDefinitionRegistrar {

    // 映射管理，避免class.forName扫描包路径
    private final Map<String, Map<String, String>> mapper = new HashMap<>();

    private final Map<String, List<EnumValue>> values = new HashMap<>();

    public EnumConfig() {
        Reflections reflections = new Reflections("com.flyfish.project");
        // 得到Resource注解的类
        Set<Class<? extends NamedEnum>> classSet = reflections.getSubTypesOf(NamedEnum.class);
        // 注入
        classSet.stream().filter(clazz -> ClassUtils.isAssignable(clazz, Enum.class)).forEach(clazz -> {
            String name = StringFormats.camel2Line(clazz.getSimpleName());
            List<EnumValue> values = Arrays.stream(clazz.getEnumConstants()).reduce(new ArrayList<>(), (result, item) -> {
                result.add(new EnumValue(((Enum<?>) item).name(), item.getName()));
                return result;
            }, (a, b) -> a);
            this.mapper.put(name, values.stream().collect(Collectors.toMap(EnumValue::getValue, EnumValue::getText)));
            this.values.put(name, values);
        });
    }

    @Override
    public void afterPropertiesSet() throws Exception {

    }
}
