package com.flyfish.framework.logging.advice;

import lombok.extern.slf4j.Slf4j;
import org.aspectj.lang.ProceedingJoinPoint;
import org.aspectj.lang.annotation.Around;
import org.aspectj.lang.annotation.Aspect;
import org.aspectj.lang.annotation.Pointcut;
import org.springframework.stereotype.Component;

/**
 * 日志切面
 *
 * @author wangyu
 */
@Aspect
@Slf4j
public class LogAdvice {

    /**
     * 切入注解
     */
    @Pointcut("@annotation( com.flyfish.framework.annotations.Operation)")
    public void pointCut() {

    }

    /**
     * 环绕切面，获取参数
     *
     * @param joinPoint 切入点
     * @return 结果
     */
    @Around("pointCut()")
    public Object process(ProceedingJoinPoint joinPoint) throws Throwable {
        Object[] args = joinPoint.getArgs();
        boolean success = false;
        try {
            Object result = joinPoint.proceed(args);
            success = true;
            return result;
        } finally {
            log.info("success: {}, {}", success,joinPoint.getSignature().getName());
        }
    }
}
