package com.flyfish.framework.controller;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.configuration.annotations.CurrentUser;
import com.flyfish.framework.domain.UserPasswordDto;
import com.flyfish.framework.domain.UserQo;
import com.flyfish.framework.domain.base.IUser;
import com.flyfish.framework.domain.po.User;
import com.flyfish.framework.service.UserService;
import com.flyfish.framework.utils.Assert;
import org.springframework.security.core.context.ReactiveSecurityContextHolder;
import org.springframework.security.crypto.password.PasswordEncoder;
import org.springframework.web.bind.annotation.*;
import reactor.core.publisher.Mono;

import javax.annotation.Resource;

@RestController
@RequestMapping("/users")
public class UserController extends BaseController<User, UserQo> {

    @Resource
    private PasswordEncoder passwordEncoder;

    /**
     * 修改密码逻辑
     *
     * @param passwordDto 密码dto
     * @param user        用户
     * @return 结果
     */
    @PutMapping("/passwords")
    public Result<Void> changePassword(@RequestBody UserPasswordDto passwordDto, @CurrentUser User user) {
        // 检查原密码
        Assert.isTrue(passwordEncoder.matches(passwordDto.getOldPassword(), user.getPassword()), "原密码不正确！");
        Assert.isTrue(!passwordEncoder.matches(passwordDto.getPassword(), user.getPassword()), "新密码和旧密码一致，输入个新的吧！");
        userContext.setUser(user);
        // 更新密码
        User updating = new User();
        updating.setId(user.getId());
        updating.setPassword(passwordEncoder.encode(passwordDto.getPassword()));
        service.updateSelectiveById(updating);
        return Result.ok();
    }

    /**
     * 获取当前用户
     *
     * @return 结果
     */
    @GetMapping("/current")
    public Mono<Result<IUser>> getCurrentUser() {
        UserService userService = getService();
        return ReactiveSecurityContextHolder.getContext()
                .map(context -> (IUser) context.getAuthentication().getPrincipal())
                .map(Result::ok);
    }
}
