package com.flyfish.framework.handler;

import com.flyfish.framework.bean.Result;
import com.flyfish.framework.transform.DataBufferTransformer;
import org.springframework.http.MediaType;
import org.springframework.http.server.reactive.ServerHttpResponse;
import org.springframework.security.authentication.BadCredentialsException;
import org.springframework.security.core.AuthenticationException;
import org.springframework.security.web.server.WebFilterExchange;
import org.springframework.security.web.server.authentication.ServerAuthenticationFailureHandler;
import reactor.core.publisher.Mono;

import java.util.HashMap;
import java.util.Map;

/**
 * 基于json的登录失败包装，详见Spring Security
 *
 * @author wangyu
 */
public class JsonAuthenticationFailureHandler implements ServerAuthenticationFailureHandler {

    private final String defaultMessage = "用户登录异常！";
    // 数据块工厂
    private final DataBufferTransformer<Result<?>> dataBufferTransformer;

    private final Map<Class<? extends AuthenticationException>, String> descriptionMap = new HashMap<>();

    public JsonAuthenticationFailureHandler(DataBufferTransformer<Result<?>> dataBufferTransformer) {
        this.dataBufferTransformer = dataBufferTransformer;
        descriptionMap.put(BadCredentialsException.class, "用户名密码不正确!");
        // todo 剩下的都在userDetailsService
    }

    /**
     * 失败时返回具体失败原因
     *
     * @param webFilterExchange 请求数据
     * @param exception         异常
     * @return 结果
     */
    @Override
    public Mono<Void> onAuthenticationFailure(WebFilterExchange webFilterExchange, AuthenticationException exception) {
        ServerHttpResponse response = webFilterExchange.getExchange().getResponse();
        response.getHeaders().setContentType(MediaType.APPLICATION_JSON);
        return response.writeWith(Mono.fromCallable(() ->
                dataBufferTransformer.transform(
                        Result.error(descriptionMap.getOrDefault(exception.getClass(), exception.getMessage()))
                )));
    }
}
